using System.Drawing.Printing;
using System.Text;


//====================================================
//       :
// https://www.interestprograms.ru
//     
//     
//====================================================

namespace WinFormsAppPrint
{
    public partial class Form1 : Form
    {
        private StreamReader? _printFile = null;
        private readonly PrintDocument _printDocument = new();

        //        .
        private string? _fileToPrint = null;

        private Font _printFont = new("Arial", 10);
        //    
        private readonly SolidBrush _printColor = new(Color.Black);

        public Form1()
        {
            InitializeComponent();

            //    ,    .NET
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);

            //    .
            foreach (EncodingInfo en in Encoding.GetEncodings())
            {
                comboBox1.Items.Add(en.Name);
            }
            comboBox1.Sorted = true;


            _printDocument.PrintPage += new PrintPageEventHandler(this.PrintDocument_PrintPage);
            _printDocument.BeginPrint += PrintDocument_BeginPrint;
            _printDocument.EndPrint += PrintDocument_EndPrint;

        }


        #region     

        private void BtnSelectFile_Click(object sender, EventArgs e)
        {
            //      .
            OpenFileDialog dlg = new()
            {
                Filter = "text files|*.txt;*.css;*.html;*.php;*.py;*.cs;*.cpp|All files (*.*)|*.*"
            };

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                _fileToPrint = dlg.FileName;
            }
        }

        #endregion


        #region    PrintDocument 

        //    .
        private void PrintDocument_BeginPrint(object sender, PrintEventArgs e)
        {
            //     .
            if (_fileToPrint != null) _printFile = new StreamReader(_fileToPrint, Encoding.GetEncoding(comboBox1.Text));
            else e.Cancel = true;
        }

        //      .
        private void PrintDocument_EndPrint(object sender, PrintEventArgs e)
        {
            //     .
            _printFile?.Close();
        }


        //    PrintDocument,
        //   -  .
        private void PrintDocument_PrintPage(object sender, PrintPageEventArgs e)
        {
            if (_printFile != null && e.Graphics is Graphics graphics)
            {
                graphics.PageUnit = GraphicsUnit.Display;
                //graphics.PageUnit = GraphicsUnit.Point;
                //graphics.PageUnit = GraphicsUnit.Millimeter;
                //graphics.PageUnit = GraphicsUnit.Inch;

                //  
                int counterLines = 0;
                //  
                string? lineText = null;
                float leftMargin = e.MarginBounds.Left;
                float topMargin = e.MarginBounds.Top;
                //       graphics.
                float fontHeight = _printFont.GetHeight(graphics);
                //      .
                float linesPerPage = e.MarginBounds.Height / fontHeight;

                Pen rectPen = new(Brushes.Red)
                {
                    Alignment = System.Drawing.Drawing2D.PenAlignment.Inset
                };

                switch (graphics.PageUnit)
                {
                    case GraphicsUnit.Display:
                        //   1
                        rectPen.Width = (1 * 100.0f) / 25.4f;
                        graphics.DrawRectangle(rectPen, 0, 0, 8.267716535433071f * 100.0f, 11.69291338582677f * 100.0f);
                        break;

                    case GraphicsUnit.Point:
                        //   1
                        rectPen.Width = (1 * 72.0f) / 25.4f;
                        linesPerPage = (e.MarginBounds.Height * (72.0f / 100.0f)) / fontHeight;
                        leftMargin = e.MarginBounds.Left * (72.0f / 100.0f);
                        topMargin = e.MarginBounds.Top * (72.0f / 100.0f);
                        graphics.DrawRectangle(rectPen, 0, 0, 8.267716535433071f * 72.0f, 11.69291338582677f * 72.0f);
                        break;

                    case GraphicsUnit.Millimeter:
                        //   1
                        rectPen.Width = 1;
                        linesPerPage = (e.MarginBounds.Height * (25.4f / 100.0f)) / fontHeight;
                        leftMargin = e.MarginBounds.Left * (25.4f / 100.0f);
                        topMargin = e.MarginBounds.Top * (25.4f / 100.0f);
                        graphics.DrawRectangle(rectPen, 0, 0, 210, 297);
                        break;

                    case GraphicsUnit.Inch:
                        //   1
                        rectPen.Width = 1 / 25.4f;
                        linesPerPage = (e.MarginBounds.Height / 100.0f) / fontHeight;
                        leftMargin = e.MarginBounds.Left / 100.0f;
                        topMargin = e.MarginBounds.Top / 100.0f;
                        graphics.DrawRectangle(rectPen, 0, 0, 8.267716535433071f, 11.69291338582677f);
                        break;
                }


                //        .
                while (counterLines < linesPerPage && ((lineText = _printFile.ReadLine()) != null))
                {
                    //         .
                    float yPos = topMargin + (counterLines * fontHeight);
                    //
                    graphics.DrawString(
                        lineText,
                        _printFont,
                        _printColor,
                        leftMargin,
                        yPos
                    );
                    //
                    counterLines++;
                }

                //    ,   .
                if (lineText != null)
                {
                    e.HasMorePages = true;
                }
                else
                {
                    //  
                    e.HasMorePages = false;
                }
            }

        }

        #endregion


        #region  

        private void BtnPreviewPrint_Click(object sender, EventArgs e)
        {
            //   
            PrintPreviewDialog dialog = new()
            {
                Document = _printDocument
            };

            dialog.ShowDialog();
        }

        private void BtnPrintDialog_Click(object sender, EventArgs e)
        {
            PrintDialog dialog = new()
            {
                Document = _printDocument,
                ShowHelp = true,
                AllowSomePages = true,
                //      Windows XP. 
                //UseEXDialog = true
            };

            dialog.HelpRequest += PrintDialog_HelpRequest;

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                _printDocument.Print();
            }
        }

        private void PrintDialog_HelpRequest(object? sender, EventArgs e)
        {
            MessageBox.Show("  !");
        }

        private void BtnPrintDocument_Click(object sender, EventArgs e)
        {
            _printDocument.Print();
        }

        #endregion


        #region  

        private void BtnPageSetup_Click(object sender, EventArgs e)
        {
            PageSetupDialog dialog = new()
            {
                // https://learn.microsoft.com/en-us/dotnet/api/system.windows.forms.pagesetupdialog.enablemetric?view=windowsdesktop-7.0
                //  .NET Framework  2.0          true
                //       ,      .
                EnableMetric = true,
                Document = _printDocument
            };
            dialog.ShowDialog();

        }

        private void BtnSelectFont_Click(object sender, EventArgs e)
        {
            FontDialog dialog = new()
            {
                Font = _printFont
            };

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                _printFont = dialog.Font;
            }
        }

        private void BtnFontColor_Click(object sender, EventArgs e)
        {
            ColorDialog dialog = new();

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                _printColor.Color = dialog.Color;
            }
        }

        #endregion




    }
}